@@ -13,23 +13,23 @@ using namespace RcppParallel;
 
 struct paraKin_p : public Worker {
   // input and others
-  const matrix4 & A;
-  const double * p;
+  uint8_t ** data;
   const size_t ncol;
   const size_t true_ncol;
-  const Ktype n;
+  const Ktype n;                  // should be (nb_snps - 1)
+  const double * p;
   const size_t sizeK;
 
   // output
   Ktype * K;
   
   // constructeurs
-  paraKin_p(matrix4 & A, const double * p) : A(A), p(p), n( (Ktype) A.nrow-1),  // ******* BEWARE THE nrow-1 !!! ***********
-        ncol(A.ncol), true_ncol(A.true_ncol), sizeK((4*true_ncol)*(4*true_ncol+1)/2) { 
+  paraKin_p(uint8_t ** data, const size_t ncol, const size_t true_ncol, const Ktype n, const double * p) :
+          data(data), ncol(ncol), true_ncol(true_ncol), n(n), p(p), sizeK((4*true_ncol)*(4*true_ncol+1)/2) { 
           K = new Ktype[sizeK];  // K is padded to a multiple of 4...
           std::fill(K, K+sizeK, 0);
         }
-  paraKin_p(paraKin_p & Q, Split) : A(Q.A), p(Q.p), n(Q.n), ncol(Q.ncol), true_ncol(Q.true_ncol), sizeK(Q.sizeK) {
+  paraKin_p(paraKin_p & Q, Split) : data(Q.data), ncol(Q.ncol), true_ncol(Q.true_ncol), n(Q.n), p(Q.p), sizeK(Q.sizeK) {
           K = new Ktype[sizeK];  
           std::fill(K, K+sizeK, 0);
         }
@@ -66,7 +66,7 @@ struct paraKin_p : public Worker {
         }
       }
 
-      uint8_t * dd = A.data[i];
+      uint8_t * dd = data[i];
       
       k = 0;
       for(size_t j1 = 0; j1 < true_ncol; j1++) {
@@ -102,7 +102,7 @@ struct paraKin_p : public Worker {
 
 
 NumericMatrix Kinship_p(XPtr<matrix4> p_A, const std::vector<double> & p, int chunk) {
-  paraKin_p X(*p_A, &p[0]);
+  paraKin_p X(p_A->data, p_A->ncol, p_A->true_ncol, (Ktype) (p_A->nrow - 1), &p[0]);
   parallelReduce(0, p_A->nrow, X, chunk);
 
   NumericMatrix Y(p_A->ncol,p_A->ncol);
@@ -124,6 +124,42 @@ NumericMatrix Kinship_p(XPtr<matrix4> p_A, const std::vector<double> & p, int ch
   return Y;
 }
 
+//[[Rcpp::export]]
+NumericMatrix Kinship_pw(XPtr<matrix4> p_A, const std::vector<double> & p, LogicalVector snps, int chunk) {
+  int nb_snps = sum(snps);
+
+  if(snps.length() != p_A->nrow || p.size() != nb_snps)
+    stop("Dimensions mismatch");
+
+  uint8_t ** data = new uint8_t * [nb_snps];
+  size_t k = 0;
+  for(size_t i = 0; i < p_A->nrow; i++) {
+    if(snps[i]) data[k++] = p_A->data[i];
+  }
+  paraKin_p X(data, p_A->ncol, p_A->true_ncol, (Ktype) (nb_snps - 1), &p[0]);
+  parallelReduce(0, nb_snps, X, chunk);
+
+  delete [] data;
+
+  NumericMatrix Y(p_A->ncol,p_A->ncol);
+  k = 0;
+  for(size_t i = 0; i < p_A->ncol; i++) {
+    for(size_t j = 0; j <= i; j++) {
+      Y(j,i) = (double) X.K[k++];
+    }
+  }
+
+  // symmetriser
+  k = 0;
+  for(size_t i = 0; i < p_A->ncol; i++) {
+    for(size_t j = 0; j <= i; j++) {
+      Y(i,j) = (double) X.K[k++]; // ou Y(j,i)
+    }
+  }
+
+  return Y;
+}
+
 
 RcppExport SEXP gg_Kinship_p(SEXP p_ASEXP, SEXP pSEXP, SEXP chunkSEXP) {
 BEGIN_RCPP
@@ -141,3 +177,16 @@ BEGIN_RCPP
 END_RCPP
 }
 
+RcppExport SEXP gg_Kinship_pw(SEXP p_ASEXP, SEXP pSEXP, SEXP snpsSEXP, SEXP chunkSEXP) {
+BEGIN_RCPP
+    Rcpp::RObject __result;
+    Rcpp::RNGScope __rngScope;
+    Rcpp::traits::input_parameter< XPtr<matrix4> >::type p_A(p_ASEXP);
+    Rcpp::traits::input_parameter< const std::vector<double>& >::type p(pSEXP);
+    Rcpp::traits::input_parameter< LogicalVector >::type snps(snpsSEXP);
+    Rcpp::traits::input_parameter< int >::type chunk(chunkSEXP);
+    __result = Rcpp::wrap(Kinship_pw(p_A, p, snps, chunk));
+    return __result;
+END_RCPP
+}
+
